<?php
/*
Plugin Name: Portugal CTT Tracking for WooCommerce
Plugin URI: 
Description: Lets you associate a tracking code with a WooCommerce order so that both the store owner and the client can track the order sent with CTT
Version: 1.1.1
Author: Webdados
Author URI: https://www.webdados.pt
Text Domain: portugal-ctt-tracking-woocommerce
Domain Path: /languages/
WC requires at least: 3.0
WC tested up to: 3.8.0
*/

/* WooCommerce CRUD ready */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/* Localization */
add_action( 'plugins_loaded', 'ctt_tracking_load_textdomain', 0 );
function ctt_tracking_load_textdomain() {
	load_plugin_textdomain( 'portugal-ctt-tracking-woocommerce' );
}

/* Check if WooCommerce is active - Get active network plugins - "Stolen" from Novalnet Payment Gateway */
function ctt_tracking_active_nw_plugins() {
	if ( !is_multisite() )
		return false;
	$ctt_tracking_activePlugins = ( get_site_option('active_sitewide_plugins' ) ) ? array_keys( get_site_option('active_sitewide_plugins' ) ) : array();
	return $ctt_tracking_activePlugins;
}
if ( in_array( 'woocommerce/woocommerce.php', (array) get_option( 'active_plugins' ) ) || in_array( 'woocommerce/woocommerce.php', (array) ctt_tracking_active_nw_plugins() ) ) {


	/* Our own order class and the main classes */
	add_action( 'plugins_loaded', 'ctt_tracking_init', 1 );
	function ctt_tracking_init() {
		if ( class_exists( 'WooCommerce' ) && version_compare( WC_VERSION, '3.0', '>=' ) ) { //We check again because WooCommerce could have "died"
			require_once( dirname( __FILE__ ) . '/includes/class-ctt-tracking.php' );
			$GLOBALS['CTT_Tracking'] = CTT_Tracking();
			/* Add settings links - This is here because inside the main class we cannot call the correct plugin_basename( __FILE__ ) */
			add_filter( 'plugin_action_links_'.plugin_basename( __FILE__ ), array( CTT_Tracking(), 'add_settings_link' ) );
		} else {
			add_action( 'admin_notices', 'admin_notices_ctt_tracking_not_active' );
		}
	}

	/* Main class */
	function CTT_Tracking() {
		return CTT_Tracking::instance(); 
	}

	
} else {


	add_action( 'admin_notices', 'admin_notices_ctt_tracking_not_active' );


}


function admin_notices_ctt_tracking_not_active() {
	?>
	<div class="notice notice-error is-dismissible">
		<p><?php _e( '<strong>Portugal CTT Tracking for WooCommerce</strong> is installed and active but <strong>WooCommerce (3.0 or above)</strong> is not.', 'portugal-ctt-tracking-woocommerce' ); ?></p>
	</div>
	<?php
}

/* If you're reading this you must know what you're doing ;-) Greetings from sunny Portugal! */

